; ------------------------------------------------------
; Sprites Collisions
; Copyright (c) 2005, Franck Charlet
; All rights reserved.
; ------------------------------------------------------
; buildblock RELEASE
;  CAPT %5\Release.bat
; buildblockend
; buildblock DEBUG
;  CAPT %5\Debug.bat
; buildblockend

; ------------------------------------------------------
; Constants
SCREEN_WIDTH     equ  800
SCREEN_HEIGHT    equ  600
SCREEN_BPP    equ  32
PADDLE_HEIGHT equ 550

MAX_SIZE    equ  128      ; Size of the textures holding the sprites

; ------------------------------------------------------
; Includes
      include  ..\Common_Src\FrameWork.asm
      include  ..\Common_Src\Bmp.asm
      include  ..\Common_Src\Textures.asm
      include  ..\Common_Src\Sprites.asm
      include  ..\Common_Src\Fonts.asm

; ------------------------------------------------------
; Variables
lFont      dd  0

Picture1    BMP  <>
Picture1_Id    dd  0
Picture2    BMP  <>
Picture2_Id    dd  0

Sprite1_Datas    SPRITE_DATAS <>
Sprite2_Datas    SPRITE_DATAS <>
Sprite1      SPRITE <>
Sprite2      SPRITE <>

My_Mouse_X    dd  0
My_Mouse_Y    dd  0
SprVX1        dd  4
SprVY1        dd  3

; ------------------------------------------------------
; Name: Initialize()
; Desc: Perform the various initializations
Initialize    proc

      invoke  glClearDepth, CDBL(1.0)
      invoke  glHint, GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST
      invoke  glClearColor, CFLT(0.2), CFLT(0.25), CFLT(0.3), CFLT(1.0)
      invoke  glEnable, GL_NORMALIZE
      invoke  glEnable, GL_CULL_FACE
; sprite1.bmp used to be loaded here.
      invoke  Load_BMP, CSTR("..\..\Media\multi030.bmp"), addr Picture1, -1
      .if  eax == FALSE
        invoke  Set_Error_Msg, CSTR("Can't load texture file")
        mov  eax, FALSE
        ret
      .endif

      invoke  Load_BMP, CSTR("..\..\Media\green002.bmp"), addr Picture2, -1
      .if  eax == FALSE
        invoke  Set_Error_Msg, CSTR("Can't load texture file")
        mov  eax, FALSE
        ret
      .endif

      invoke  Init_Textures_Context

      ; Create the visual datas for the sprites
      mov  Sprite1_Datas.Sprite_Width, CMEM(Picture1._Width)
      mov  Sprite1_Datas.Sprite_Height, CMEM(Picture1._Height)
      mov  Sprite1_Datas.Sprite_Datas, CMEM(Picture1._Datas)
      mov  Sprite1_Datas.Texture_Size, MAX_SIZE      ; Must be a standard texture size
      mov  Sprite1_Datas.Color_Key, 0ff00ffh      ; Background color
      mov  Sprite1_Datas.Sprite_Pic_Width, CMEM(Picture1._Width)
      mov  Sprite1_Datas.Sprite_Pic_Height, CMEM(Picture1._Height)
      invoke  Create_Sprite_Datas, addr Sprite1_Datas
      .if  eax == FALSE
        invoke  Set_Error_Msg, CSTR("Can't create sprite")
        mov  eax, FALSE
        ret
      .endif

      mov  Sprite2_Datas.Sprite_Width, CMEM(Picture2._Width)
      mov  Sprite2_Datas.Sprite_Height, CMEM(Picture2._Height)
      mov  Sprite2_Datas.Sprite_Datas, CMEM(Picture2._Datas)
      mov  Sprite2_Datas.Texture_Size, MAX_SIZE      ; Must be a standard texture size
      mov  Sprite2_Datas.Color_Key, 0ff00ffh      ; Background color
      mov  Sprite2_Datas.Sprite_Pic_Width, CMEM(Picture2._Width)
      mov  Sprite2_Datas.Sprite_Pic_Height, CMEM(Picture2._Height)
      invoke  Create_Sprite_Datas, addr Sprite2_Datas
      .if  eax == FALSE
        invoke  Set_Error_Msg, CSTR("Can't create sprite")
        mov  eax, FALSE
        ret
      .endif

      ; Assign them to our sprites
      invoke  Create_Sprite, addr Sprite1, addr Sprite1_Datas
      invoke  Create_Sprite, addr Sprite2, addr Sprite2_Datas

      ; Create the fonts
      invoke  Create_Font_Bitmap, CSTR("Courier New"), 32, FW_BOLD
      mov  lFont, eax
      .if  eax == 0
        invoke  Set_Error_Msg, CSTR("Can't create font")
        mov  eax, FALSE
        ret
      .endif

      mov  eax,TRUE
      ret
Initialize    endp

; ------------------------------------------------------
; Name: Deinitialize()
; Desc: Free allocated resources
Deinitialize    proc
      invoke  Destroy_Sprite_Datas, addr Sprite1_Datas
      invoke  Destroy_Sprite_Datas, addr Sprite2_Datas

      invoke  Destroy_Font, lFont
      invoke  Unload_BMP, addr Picture2
      invoke  Unload_BMP, addr Picture1
      ret
Deinitialize    endp

; ------------------------------------------------------
; Name: Update()
; Desc: Update the scene
Update      proc  FrameRate:real4

      ; Update scene here
      invoke  DInput_Clamp_Mouse, Mouse_X, addr My_Mouse_X, 0, SCREEN_WIDTH - 1
      invoke  DInput_Clamp_Mouse, Mouse_Y, addr My_Mouse_Y, 0, SCREEN_HEIGHT - 1

;SprX1         dd  0
;SprVX1        dd  4
      mov eax, SprVX1             ; eax = velocity
      add Sprite2.X, eax              ; SprX1 = SprX1 + velocity
      mov eax, SCREEN_WIDTH
      sub eax, Picture2._Width
      cmp Sprite2.X, eax          ; Compare SprX1 with SCREEN_WIDTH
      jl nothitx1                 ; If less than width, fine did not hit this boundary
      mov Sprite2.X, eax          ; otherwise, set equal to maximum value
      mov eax, SprVX1             ; eax = velocity
      neg eax                     ; change direction
      mov SprVX1, eax             ; set change in direction
      jmp nothitx2                ; bypass next test
nothitx1:
      cmp Sprite2.X, 0                ; Test with left side of screen boundary
      jg nothitx2                 ; if greater than, fine did not hit this boundary
      mov Sprite2.X, 0                ; otherwise, set equal to minimum value
      mov eax, SprVX1             ; eax = velocity
      neg eax                     ; change direction
      mov SprVX1, eax             ; set change in direction
nothitx2:

      mov eax, SprVY1
      add Sprite2.Y, eax
      mov eax, SCREEN_HEIGHT
      sub eax, Picture2._Height
      cmp Sprite2.Y, eax
      jl nothity1
      mov Sprite2.Y, eax
      mov eax, SprVY1
      neg eax
      mov SprVY1, eax
      jmp nothity2
nothity1:
      cmp Sprite2.Y, 0
      jg nothity2
      mov Sprite2.Y, 0
      mov eax, SprVY1
      neg eax
      mov SprVY1, eax
nothity2:

      invoke  DInput_Get_Key, DIK_ESCAPE
      test  eax, eax
      jz  Esc_Pressed
      invoke  Terminate_Application
Esc_Pressed:    ret
Update      endp

; ------------------------------------------------------
; Name: Draw()
; Desc: Draw the scene
Draw      proc
      local  Collide:dword

      invoke  glClear, GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT or GL_STENCIL_BUFFER_BIT

      ; Draw scene here
      invoke  Enter_2D_Mode, INT2FLT(SCREEN_WIDTH), INT2FLT(SCREEN_HEIGHT)
        invoke  glEnable, GL_BLEND
        invoke  glBlendFunc, GL_DST_ALPHA, GL_ONE_MINUS_SRC_ALPHA

        invoke  glColor3f, CFLT(1.0), CFLT(1.0), CFLT(1.0)

        ; Sprite 1 controlled by the mouse
;        mov  ebx, My_Mouse_Y
        mov  eax, My_Mouse_X
        mov  Sprite1.X, eax
        add eax, Picture1._Width
        cmp eax, SCREEN_WIDTH
        jl fine
        mov eax, SCREEN_WIDTH
        sub eax, Picture1._Width
        mov Sprite1.X, eax
fine:
        mov Sprite1.Y, PADDLE_HEIGHT
        invoke  Display_Sprite, addr Sprite1

        ; Sprite 2 centered on screen
;        mov  eax, SCREEN_WIDTH
;        sub  eax, Picture1._Width
;        shr  eax, 1
;        mov  Sprite2.X, eax
;        mov  ebx, SCREEN_HEIGHT
;        sub  ebx, Picture1._Height
;        shr  ebx, 1
;        mov  Sprite2.Y, ebx
        invoke  Display_Sprite, addr Sprite2

        invoke  glDisable, GL_BLEND

        invoke  Check_Collision, addr Sprite1, addr Sprite2
        .if  eax == TRUE
          mov eax, SprVY1             ; eax = velocity
          neg eax                     ; change direction
          mov SprVY1, eax             ; set change in direction

          invoke  glTranslatef, CFLT(0.0), CFLT(0.0), CFLT(-1.0)
          invoke  glColor4f, CFLT(1.0), CFLT(0.8), CFLT(0.2), CFLT(0.0)
          invoke  Display_2D_Text, INT2FLT(10), INT2FLT(502), lFont, CSTR("COLLISION JEFF"), 14
        .endif
      invoke  Leave_2d_Mode

      ret
Draw      endp

end start